@@ -45,6 +45,9 @@ void init_collage_mode(void)
 
 	mode = "collage";
 
+	weprintf("the --collage option (aka collage mode) is deprecated\n"
+			"and will be removed soon\n");
+
 	/* Use bg image dimensions for default size */
 	if (opt.bg && opt.bg_file) {
 		if (!strcmp(opt.bg_file, "trans"))
@@ -63,7 +66,7 @@ void init_collage_mode(void)
 		if (bg_im) {
 			if (opt.verbose)
 				fputs(PACKAGE
-					" - No size restriction specified for collage.\n"
+					": No size restriction specified for collage.\n"
 					" You did specify a background however, so the\n"
 					" collage size has defaulted to the size of the image\n",
 					stdout);
@@ -72,7 +75,7 @@ void init_collage_mode(void)
 		} else {
 			if (opt.verbose)
 				fputs(PACKAGE
-					" - No size restriction specified for collage.\n"
+					": No size restriction specified for collage.\n"
 					" - For collage mode, you need to specify width and height.\n"
 					" Using defaults (width 800, height 600)\n",
 					stdout);
@@ -196,7 +199,7 @@ void init_collage_mode(void)
 
 			tw = gib_imlib_image_get_width(im_main);
 			th = gib_imlib_image_get_height(im_main);
-			fprintf(stdout, PACKAGE " - File saved as %s\n", output_buf);
+			fprintf(stdout, PACKAGE ": File saved as %s\n", output_buf);
 			fprintf(stdout,
 				"    - Image is %dx%d pixels and contains %d thumbnails\n",
 				tw, th, (tw / opt.thumb_w) * (th / opt.thumb_h));
