@@ -1,6 +1,7 @@
 /* feh.h
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -92,11 +93,23 @@ enum bgmode_type { BG_MODE_NONE = 0, BG_MODE_TILE, BG_MODE_CENTER,
 	BG_MODE_SCALE, BG_MODE_FILL, BG_MODE_MAX
 };
 
+enum zoom_mode { ZOOM_MODE_FILL = 1, ZOOM_MODE_MAX };
+
+enum text_bg { TEXT_BG_CLEAR = 0, TEXT_BG_TINTED };
+
 enum slide_change { SLIDE_NEXT, SLIDE_PREV, SLIDE_RAND, SLIDE_FIRST, SLIDE_LAST,
 	SLIDE_JUMP_FWD,
 	SLIDE_JUMP_BACK
 };
 
+enum image_bg { IMAGE_BG_CHECKS = 0, IMAGE_BG_BLACK, IMAGE_BG_WHITE };
+
+#define INPLACE_EDIT_FLIP   -1
+#define INPLACE_EDIT_MIRROR -2
+
+#define ZOOM_MIN 0.002
+#define ZOOM_MAX 2000
+
 typedef void (*sighandler_t) (int);
 
 int feh_main_iteration(int block);
@@ -117,12 +130,15 @@ void init_unloadables_mode(void);
 void feh_clean_exit(void);
 int feh_load_image(Imlib_Image * im, feh_file * file);
 void show_mini_usage(void);
-void slideshow_change_image(winwidget winwid, int change);
+void slideshow_change_image(winwidget winwid, int change, int render);
 void slideshow_pause_toggle(winwidget w);
 char *slideshow_create_name(feh_file * file);
+void init_keyevents(void);
+void init_buttonbindings(void);
 void feh_event_handle_keypress(XEvent * ev);
 void feh_action_run(feh_file * file, char *action);
 char *feh_printf(char *str, feh_file * file);
+void im_weprintf(winwidget w, char *fmt, ...);
 void feh_draw_zoom(winwidget w);
 void feh_draw_checks(winwidget win);
 void cb_slide_timer(void *data);
@@ -130,20 +146,22 @@ void cb_reload_timer(void *data);
 char *feh_http_load_image(char *url);
 int feh_load_image_char(Imlib_Image * im, char *filename);
 void feh_draw_filename(winwidget w);
+#ifdef HAVE_LIBEXIF
+void feh_draw_exif(winwidget w);
+#endif
 void feh_draw_actions(winwidget w);
 void feh_draw_caption(winwidget w);
 void feh_draw_info(winwidget w);
+void feh_draw_errstr(winwidget w);
 void feh_display_status(char stat);
 void real_loadables_mode(int loadable);
 void feh_reload_image(winwidget w, int resize, int force_new);
 void feh_filelist_image_remove(winwidget winwid, char do_delete);
-char *feh_strip_hostname(char *url);
-struct hostent *feh_gethostbyname(const char *name);
 void slideshow_save_image(winwidget win);
-void feh_edit_inplace_orient(winwidget w, int orientation);
-void feh_edit_inplace_lossless_rotate(winwidget w, int orientation);
+void feh_edit_inplace(winwidget w, int orientation);
+void feh_edit_inplace_lossless(winwidget w, int orientation);
 gib_list *feh_wrap_string(char *text, int wrap_width, Imlib_Font fn, gib_style * style);
-char *build_caption_filename(feh_file * file);
+char *build_caption_filename(feh_file * file, short create_dir);
 gib_list *feh_list_jump(gib_list * root, gib_list * l, int direction, int num);
 
 /* Imlib stuff */
