@@ -24,6 +24,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
 
+#ifdef HAVE_LIBEXIF
+#include <libexif/exif-data.h>
+#endif
+
 #include "feh.h"
 #include "filelist.h"
 #include "options.h"
@@ -65,6 +69,10 @@ void feh_file_free(feh_file * file)
 		free(file->caption);
 	if (file->info)
 		feh_file_info_free(file->info);
+#ifdef HAVE_LIBEXIF
+	if (file->info)
+		exif_data_unref(file->ed);
+#endif		
 	free(file);
 	return;
 }
@@ -118,6 +126,33 @@ int file_selector_all(const struct dirent *unused __attribute__((unused)))
   return 1;
 }
 
+static void feh_print_stat_error(char *path)
+{
+	if (opt.quiet)
+		return;
+
+	switch (errno) {
+	case ENOENT:
+	case ENOTDIR:
+		weprintf("%s does not exist - skipping", path);
+		break;
+	case ELOOP:
+		weprintf("%s - too many levels of symbolic links - skipping", path);
+		break;
+	case EACCES:
+		weprintf("you don't have permission to open %s - skipping", path);
+		break;
+	case EOVERFLOW:
+		weprintf("Cannot open %s - EOVERFLOW.\n"
+			"Recompile with stat64=1 to fix this", path);
+		break;
+	default:
+		weprintf("couldn't open %s", path);
+		break;
+	}
+}
+
+
 /* Recursive */
 void add_file_to_filelist_recursively(char *origpath, unsigned char level)
 {
@@ -157,27 +192,7 @@ void add_file_to_filelist_recursively(char *origpath, unsigned char level)
 
 	errno = 0;
 	if (stat(path, &st)) {
-		if (!opt.quiet) {
-			switch (errno) {
-			case ENOENT:
-			case ENOTDIR:
-				weprintf("%s does not exist - skipping", path);
-				break;
-			case ELOOP:
-				weprintf("%s - too many levels of symbolic links - skipping", path);
-				break;
-			case EACCES:
-				weprintf("you don't have permission to open %s - skipping", path);
-				break;
-			case EOVERFLOW:
-				weprintf("Cannot open %s - EOVERFLOW.\n"
-					"Recompile with stat64=1 to fix this");
-				break;
-			default:
-				weprintf("couldn't open %s", path);
-				break;
-			}
-		}
+		feh_print_stat_error(path);
 		free(path);
 		return;
 	}
@@ -297,33 +312,14 @@ int feh_file_info_load(feh_file * file, Imlib_Image im)
 
 	errno = 0;
 	if (stat(file->filename, &st)) {
-		if (!opt.quiet) {
-			switch (errno) {
-			case ENOENT:
-			case ENOTDIR:
-				weprintf("%s does not exist - skipping", file->filename);
-				break;
-			case ELOOP:
-				weprintf("%s - too many levels of symbolic links - skipping", file->filename);
-				break;
-			case EACCES:
-				weprintf("you don't have permission to open %s - skipping", file->filename);
-				break;
-			default:
-				weprintf("couldn't open %s ", file->filename);
-				break;
-			}
-		}
+		feh_print_stat_error(file->filename);
 		return(1);
 	}
 
 	if (im)
 		im1 = im;
-	else if (!feh_load_image(&im1, file) || !im1) {
-		if (!opt.quiet)
-			weprintf("couldn't load %s", file->filename);
+	else if (!feh_load_image(&im1, file) || !im1)
 		return(1);
-	}
 
 	file->info = feh_file_info_new();
 
@@ -460,10 +456,23 @@ gib_list *feh_read_filelist(char *filename)
 	FILE *fp;
 	gib_list *list = NULL;
 	char s[1024], s1[1024];
+	Imlib_Image tmp_im;
+	struct stat st;
 
 	if (!filename)
 		return(NULL);
 
+	/*
+	 * feh_load_image will fail horribly if filename is not seekable
+	 */
+	if (!stat(filename, &st) && S_ISREG(st.st_mode) &&
+			feh_load_image_char(&tmp_im, filename)) {
+		weprintf("Filelist file %s is an image, refusing to use it.\n"
+			"Did you mix up -f and -F?", filename);
+		opt.filelistfile = NULL;
+		return NULL;
+	}
+
 	errno = 0;
 	if ((fp = fopen(filename, "r")) == NULL) {
 		/* return quietly, as it's okay to specify a filelist file that doesn't
