@@ -340,6 +340,9 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		{"cycle-once"    , 0, 0, 224},
 		{"no-xinerama"   , 0, 0, 225},
 		{"draw-tinted"   , 0, 0, 229},
+#ifdef HAVE_LIBEXIF
+		{"draw-exif"     , 0, 0, 223},
+#endif
 
 		{"output"        , 1, 0, 'o'},
 		{"output-only"   , 1, 0, 'O'},
@@ -685,6 +688,11 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		case 'P':
 			opt.cache_thumbnails = 1;
 			break;
+#ifdef HAVE_LIBEXIF
+		case 223:
+			opt.draw_exif = 1;
+			break;
+#endif
 		case 224:
 			opt.cycle_once = 1;
 			break;
@@ -780,6 +788,10 @@ static void show_version(void)
 		"debug "
 #endif
 
+#ifdef HAVE_LIBEXIF
+		"exif "
+#endif
+
 #if _FILE_OFFSET_BITS == 64
 		"stat64 "
 #endif
@@ -795,9 +807,13 @@ static void show_version(void)
 
 void show_mini_usage(void)
 {
-	fputs(PACKAGE " - No loadable images specified.\n"
-			"Use " PACKAGE " --help for detailed usage information\n",
-			stderr);
+	fputs(PACKAGE ": No loadable images specified.\n"
+#ifdef INCLUDE_HELP
+		"See '" PACKAGE " --help' or 'man " PACKAGE "' for detailed usage information\n",
+#else
+		"See 'man " PACKAGE "' for detailed usage information\n",
+#endif
+		stderr);
 	exit(1);
 }
 
