@@ -31,10 +31,8 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "thumbnail.h"
 #include "md5.h"
 #include "feh_png.h"
+#include "index.h"
 
-static char *create_index_dimension_string(int w, int h);
-static char *create_index_size_string(char *file);
-static char *create_index_title_string(int num, int w, int h);
 static gib_list *thumbnails = NULL;
 
 static thumbmode_data td;
@@ -65,15 +63,15 @@ void init_thumbnail_mode(void)
 	unsigned char trans_bg = 0;
 	int title_area_h = 0;
 	int tw = 0, th = 0;
-	int fw_name, fw_size, fw_dim, fh;
+	int fw, fh;
 	int thumbnailcount = 0;
 	feh_file *file = NULL;
 	gib_list *l, *last = NULL;
-	int lines;
+	int lineno;
 	int index_image_width, index_image_height;
-	int x_offset_name = 0, x_offset_dim = 0, x_offset_size = 0;
 	char *s;
 	unsigned int thumb_counter = 0;
+	gib_list *line, *lines;
 
 	/* initialize thumbnail mode data */
 	td.im_main = NULL;
@@ -104,6 +102,9 @@ void init_thumbnail_mode(void)
 		int fh, fw;
 
 		td.font_title = gib_imlib_load_font(opt.title_font);
+		if (!td.font_title)
+			td.font_title = gib_imlib_load_font(DEFAULT_FONT_TITLE);
+
 		gib_imlib_get_text_size(td.font_title, "W", NULL, &fw, &fh,
 				IMLIB_TEXT_TO_RIGHT);
 		title_area_h = fh + 4;
@@ -116,9 +117,8 @@ void init_thumbnail_mode(void)
 	/* Work out how tall the font is */
 	gib_imlib_get_text_size(td.font_main, "W", NULL, &tw, &th,
 			IMLIB_TEXT_TO_RIGHT);
-	/* For now, allow room for the right number of lines with small gaps */
-	td.text_area_h = ((th + 2) * (opt.index_show_name + opt.index_show_size +
-				opt.index_show_dim)) + 5;
+	get_index_string_dim(NULL, td.font_main, &fw, &fh);
+	td.text_area_h = fh + 5;
 
 	/* This includes the text area for index data */
 	td.thumb_tot_h = opt.thumb_h + td.text_area_h;
@@ -261,34 +261,18 @@ void init_thumbnail_mode(void)
 
 			td.text_area_w = opt.thumb_w;
 			/* Now draw on the info text */
-			if (opt.index_show_name) {
-				gib_imlib_get_text_size(td.font_main, file->name, NULL,
-						&fw_name, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw_name > td.text_area_w)
-					td.text_area_w = fw_name;
-			}
-			if (opt.index_show_dim) {
-				gib_imlib_get_text_size(td.font_main,
-							create_index_dimension_string(ww, hh),
-							NULL, &fw_dim, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw_dim > td.text_area_w)
-					td.text_area_w = fw_dim;
-			}
-			if (opt.index_show_size) {
-				gib_imlib_get_text_size(td.font_main,
-							create_index_size_string(file->filename),
-							NULL, &fw_size, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw_size > td.text_area_w)
-					td.text_area_w = fw_size;
+			if (opt.index_info) {
+				get_index_string_dim(file, td.font_main, &fw, &fh);
+				if (fw > td.text_area_w)
+					td.text_area_w = fw;
+				if (fh > td.text_area_h) {
+					td.text_area_h = fh + 5;
+					td.thumb_tot_h = opt.thumb_h + td.text_area_h;
+				}
 			}
 			if (td.text_area_w > opt.thumb_w)
 				td.text_area_w += 5;
 
-			/* offsets for centering text */
-			x_offset_name = (td.text_area_w - fw_name) / 2;
-			x_offset_dim = (td.text_area_w - fw_dim) / 2;
-			x_offset_size = (td.text_area_w - fw_size) / 2;
-
 			if (td.vertical) {
 				if (td.text_area_w > td.max_column_w)
 					td.max_column_w = td.text_area_w;
@@ -329,28 +313,23 @@ void init_thumbnail_mode(void)
 
 			gib_imlib_free_image_and_decache(im_thumb);
 
-			lines = 0;
-			if (opt.index_show_name)
-				gib_imlib_text_draw(td.im_main,
-						td.font_main, NULL,
-						x + x_offset_name,
-						y + opt.thumb_h + (lines++ * (th + 2)) + 2,
-						file->name, IMLIB_TEXT_TO_RIGHT,
-						255, 255, 255, 255);
-			if (opt.index_show_dim)
-				gib_imlib_text_draw(td.im_main,
-						td.font_main, NULL,
-						x + x_offset_dim,
-						y + opt.thumb_h + (lines++ * (th + 2)) + 2,
-						create_index_dimension_string(orig_w, orig_h),
-						IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
-			if (opt.index_show_size)
-				gib_imlib_text_draw(td.im_main,
-						td.font_main, NULL,
-						x + x_offset_size,
-						y + opt.thumb_h + (lines++ * (th + 2)) + 2,
-						create_index_size_string(file->filename),
-						IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
+			lineno = 0;
+			if (opt.index_info) {
+				line = lines = feh_wrap_string(create_index_string(file),
+						opt.thumb_w * 3, td.font_main, NULL);
+
+				while (line) {
+					gib_imlib_get_text_size(td.font_main, (char *) line -> data,
+							NULL, &fw, &fh, IMLIB_TEXT_TO_RIGHT);
+					gib_imlib_text_draw(td.im_main, td.font_main, NULL,
+							x + ((td.text_area_w - fw) >> 1),
+							y + opt.thumb_h + (lineno++ * (th + 2)) + 2,
+							(char *) line->data,
+							IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
+					line = line->next;
+				}
+				gib_list_free_and_data(lines);
+			}
 
 			if (td.vertical)
 				y += td.thumb_tot_h;
@@ -389,6 +368,9 @@ void init_thumbnail_mode(void)
 		fy = index_image_height - fh - 2;
 		gib_imlib_text_draw(td.im_main, td.font_title, NULL, fx,
 				fy, s, IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
+
+		if (opt.display)
+			winwidget_render_image(winwid, 0, 1);
 	}
 
 	if (opt.output && opt.output_file) {
@@ -418,40 +400,6 @@ void init_thumbnail_mode(void)
 	return;
 }
 
-static char *create_index_size_string(char *file)
-{
-	static char str[50];
-	int size = 0;
-	double kbs = 0.0;
-	struct stat st;
-
-	if (stat(file, &st))
-		kbs = 0.0;
-	else {
-		size = st.st_size;
-		kbs = (double) size / 1000;
-	}
-
-	snprintf(str, sizeof(str), "%.2fKb", kbs);
-	return(str);
-}
-
-static char *create_index_dimension_string(int w, int h)
-{
-	static char str[50];
-
-	snprintf(str, sizeof(str), "%dx%d", w, h);
-	return(str);
-}
-
-static char *create_index_title_string(int num, int w, int h)
-{
-	static char str[50];
-
-	snprintf(str, sizeof(str), PACKAGE " index - %d thumbnails, %d by %d pixels", num, w, h);
-	return(str);
-}
-
 feh_thumbnail *feh_thumbnail_new(feh_file * file, int x, int y, int w, int h)
 {
 	feh_thumbnail *thumb;
@@ -527,24 +475,20 @@ void feh_thumbnail_mark_removed(feh_file * file, int deleted)
 	if (thumb) {
 		w = winwidget_get_first_window_of_type(WIN_TYPE_THUMBNAIL);
 		if (w) {
-			td.font_main = imlib_load_font(DEFAULT_FONT_TITLE);
+			int tw, th;
 			if (deleted)
 				gib_imlib_image_fill_rectangle(w->im, thumb->x, thumb->y,
 						thumb->w, thumb->h, 255, 0, 0, 150);
 			else
 				gib_imlib_image_fill_rectangle(w->im, thumb->x, thumb->y,
 						thumb->w, thumb->h, 0, 0, 255, 150);
-			if (td.font_main) {
-				int tw, th;
 
-				gib_imlib_get_text_size(td.font_main, "X", NULL, &tw, &th,
-						IMLIB_TEXT_TO_RIGHT);
-				gib_imlib_text_draw(w->im, td.font_main, NULL,
-						thumb->x + ((thumb->w - tw) / 2),
-						thumb->y + ((thumb->h - th) / 2), "X",
-						IMLIB_TEXT_TO_RIGHT, 205, 205, 50, 255);
-			} else
-				weprintf(DEFAULT_FONT_TITLE);
+			gib_imlib_get_text_size(td.font_main, "X", NULL, &tw, &th,
+					IMLIB_TEXT_TO_RIGHT);
+			gib_imlib_text_draw(w->im, td.font_main, NULL,
+					thumb->x + ((thumb->w - tw) / 2),
+					thumb->y + ((thumb->h - th) / 2), "X",
+					IMLIB_TEXT_TO_RIGHT, 205, 205, 50, 255);
 			winwidget_render_image(w, 0, 1);
 		}
 		thumb->exists = 0;
@@ -554,175 +498,38 @@ void feh_thumbnail_mark_removed(feh_file * file, int deleted)
 
 void feh_thumbnail_calculate_geometry(void)
 {
-	gib_list *l;
-	feh_file *file;
-
-	int x = 0, y = 0;
-	int fw, fh;
-
 	if (!opt.limit_w && !opt.limit_h) {
 		if (td.im_bg) {
-			if (opt.verbose)
-				fputs(PACKAGE
-					" - No size restriction specified for index.\n"
-					" You did specify a background however, so the\n"
-					" index size has defaulted to the size of the image\n",
-					stdout);
 			opt.limit_w = td.bg_w;
 			opt.limit_h = td.bg_h;
-		} else {
-			if (opt.verbose)
-				fputs(PACKAGE
-					" - No size restriction specified for index.\n"
-					" Using defaults (width limited to 640)\n",
-					stdout);
-			opt.limit_w = 640;
-		}
+		} else
+			opt.limit_w = 800;
 	}
 
 	/* Here we need to whiz through the files, and look at the filenames and
 	   info in the selected font, work out how much space we need, and
 	   calculate the size of the image we will require */
 
-	if (opt.limit_w && opt.limit_h) {
-		int rec_h = 0;
-
+	if (opt.limit_w) {
 		td.w = opt.limit_w;
-		td.h = opt.limit_h;
 
-		/* Work out if this is big enough, and give a warning if not */
+		index_calculate_height(td.font_main, td.w, &td.h, &td.thumb_tot_h);
 
-		/* Pretend we are limiting width by that specified, loop through, and
-		   see it we fit in the height specified. If not, continue the loop,
-		   and recommend the final value instead. Carry on and make the index
-		   anyway. */
-
-		for (l = filelist; l; l = l->next) {
-			file = FEH_FILE(l->data);
-			td.text_area_w = opt.thumb_w;
-			if (opt.index_show_name) {
-				gib_imlib_get_text_size(td.font_main, file->name, NULL,
-						&fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-			if (opt.index_show_dim) {
-				gib_imlib_get_text_size(td.font_main,
-						create_index_dimension_string(1000, 1000),
-						NULL, &fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-			if (opt.index_show_size) {
-				gib_imlib_get_text_size(td.font_main,
-						create_index_size_string(file->filename),
-						NULL, &fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-			if (td.text_area_w > opt.thumb_w)
-				td.text_area_w += 5;
-
-			if ((x > td.w - td.text_area_w)) {
-				x = 0;
-				y += td.thumb_tot_h;
-			}
-
-			x += td.text_area_w;
-		}
-		rec_h = y + td.thumb_tot_h;
-
-		if (td.h < rec_h) {
-			weprintf
-			    ("The image size you specified (%d by %d) is not large\n"
-			     "enough to hold all the thumnails you specified (%d). To fit all\n"
-			     "the thumnails, either decrease their size, choose a smaller font,\n"
-			     "or use a larger image (may I recommend %d by %d?)",
-			     opt.limit_w, opt.limit_h, filelist_len, opt.limit_w, rec_h);
+		if (opt.limit_h) {
+			if (td.h> opt.limit_h)
+				weprintf(
+					"The image size you specified (%dx%d) is not large\n"
+					"enough to hold all %d thumbnails. To fit all\n"
+					"the thumnails, either decrease their size, choose a smaller font,\n"
+					"or use a larger image (like %dx%d)",
+					opt.limit_w, opt.limit_h, filelist_len, opt.limit_w, td.h);
+			td.h = opt.limit_h;
 		}
 	} else if (opt.limit_h) {
 		td.vertical = 1;
 		td.h = opt.limit_h;
-		/* calc w */
-		for (l = filelist; l; l = l->next) {
-			file = FEH_FILE(l->data);
-			td.text_area_w = opt.thumb_w;
-			/* Calc width of text */
-			if (opt.index_show_name) {
-				gib_imlib_get_text_size(td.font_main, file->name, NULL,
-						&fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-			if (opt.index_show_dim) {
-				gib_imlib_get_text_size(td.font_main,
-							create_index_dimension_string(1000, 1000),
-							NULL, &fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-			if (opt.index_show_size) {
-				gib_imlib_get_text_size(td.font_main,
-							create_index_size_string(file->filename),
-							NULL, &fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-			if (td.text_area_w > opt.thumb_w)
-				td.text_area_w += 5;
 
-			if (td.text_area_w > td.max_column_w)
-				td.max_column_w = td.text_area_w;
-
-			if ((y > td.h - td.thumb_tot_h)) {
-				y = 0;
-				x += td.max_column_w;
-				td.max_column_w = 0;
-			}
-
-			y += td.thumb_tot_h;
-		}
-		td.w = x + td.text_area_w;
-		td.max_column_w = 0;
-	} else if (opt.limit_w) {
-		td.w = opt.limit_w;
-		/* calc h */
-
-		for (l = filelist; l; l = l->next) {
-			file = FEH_FILE(l->data);
-			td.text_area_w = opt.thumb_w;
-			if (opt.index_show_name) {
-				gib_imlib_get_text_size(td.font_main, file->name, NULL,
-						&fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-			if (opt.index_show_dim) {
-				gib_imlib_get_text_size(td.font_main,
-						create_index_dimension_string(1000, 1000),
-						NULL, &fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-			if (opt.index_show_size) {
-				gib_imlib_get_text_size(td.font_main,
-						create_index_size_string(file->filename),
-						NULL, &fw, &fh, IMLIB_TEXT_TO_RIGHT);
-				if (fw > td.text_area_w)
-					td.text_area_w = fw;
-			}
-
-			if (td.text_area_w > opt.thumb_w)
-				td.text_area_w += 5;
-
-			if ((x > td.w - td.text_area_w)) {
-				x = 0;
-				y += td.thumb_tot_h;
-			}
-
-			x += td.text_area_w;
-		}
-		td.h = y + td.thumb_tot_h;
+		index_calculate_width(td.font_main, &td.w, td.h, &td.thumb_tot_h);
 	}
 }
 
@@ -999,6 +806,13 @@ inline void feh_thumbnail_show_selected()
 		feh_thumbnail_show_fullsize(td.selected->file);
 }
 
+inline feh_file* feh_thumbnail_get_selected_file()
+{
+	if (td.selected)
+		return td.selected->file;
+	return NULL;
+}
+
 int feh_thumbnail_setup_thumbnail_dir(void)
 {
 	int status = 0;
@@ -1019,14 +833,17 @@ int feh_thumbnail_setup_thumbnail_dir(void)
 
 			if (stat(dir_thumbnails, &sb) != 0) {
 				if (mkdir(dir_thumbnails, 0700) == -1)
-					weprintf("unable to create %s directory", dir_thumbnails);
+					weprintf("unable to create directory %s", dir_thumbnails);
 			}
 
+			free(dir_thumbnails);
+
 			if (mkdir(dir, 0700) == -1)
-				weprintf("unable to create %s directory", dir);
+				weprintf("unable to create directory %s", dir);
 			else
 				status = 1;
 		}
+		free(dir);
 	}
 
 	return status;
